' I specifically did not import System.IO to avoid any potential name collisions
Imports System.Runtime.InteropServices
Imports System.Collections.Specialized
Imports System.Security.AccessControl
Imports System.ComponentModel

'
' The replacement System.IO.FileSystemInfo class that can handle files/paths
' that are greater then 260 characters in length.  This is the base
' class for FileInfo and DirectoryInfo
'
Public MustInherit Class FileSystemInfoEx

#Region "API Region"

    Const MAX_PATH As Integer = 260
    Protected Const INVALID_HANDLE_VALUE As Integer = -1

    'typedef struct _FILETIME {
    '  DWORD dwLowDateTime;
    '  DWORD dwHighDateTime;
    '} FILETIME,  *PFILETIME;
    <StructLayout(LayoutKind.Sequential)> _
    Protected Friend Structure FILETIME
        Dim dwLowDateTime As UInt32
        Dim dwHighDateTime As UInt32
    End Structure

    'typedef struct _WIN32_FIND_DATA {
    '  DWORD dwFileAttributes;
    '  FILETIME ftCreationTime;
    '  FILETIME ftLastAccessTime;
    '  FILETIME ftLastWriteTime;
    '  DWORD nFileSizeHigh;
    '  DWORD nFileSizeLow;
    '  DWORD dwReserved0;
    '  DWORD dwReserved1;
    '  TCHAR cFileName[MAX_PATH];
    '  TCHAR cAlternateFileName[14];
    '} WIN32_FIND_DATA,  *PWIN32_FIND_DATA,  *LPWIN32_FIND_DATA;
    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Unicode)> _
    Protected Friend Structure WIN32_FIND_DATA
        Dim dwFileAttributes As Integer
        Dim ftCreationTime As FILETIME
        Dim ftLastAccessTime As FILETIME
        Dim ftLastWriteTime As FILETIME
        Dim nFileSizeHigh As UInt32
        Dim nFileSizeLow As UInt32
        Dim dwReserved0 As Integer
        Dim dwReserved1 As Integer
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=MAX_PATH)> Dim cFileName As String
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=14)> Dim cAlternateFileName As String
    End Structure

    'typedef struct _WIN32_FILE_ATTRIBUTE_DATA {
    '  DWORD dwFileAttributes;
    '  FILETIME ftCreationTime;
    '  FILETIME ftLastAccessTime;
    '  FILETIME ftLastWriteTime;
    '  DWORD nFileSizeHigh;
    '  DWORD nFileSizeLow;
    '} WIN32_FILE_ATTRIBUTE_DATA,  *LPWIN32_FILE_ATTRIBUTE_DATA;
    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Unicode)> _
    Private Structure WIN32_FILE_ATTRIBUTE_DATA
        Dim dwFileAttributes As Integer
        Dim ftCreationTime As FILETIME
        Dim ftLastAccessTime As FILETIME
        Dim ftLastWriteTime As FILETIME
        Dim nFileSizeHigh As UInt32
        Dim nFileSizeLow As UInt32
    End Structure

    'BOOL GetFileAttributesEx(
    '  LPCTSTR lpFileName,
    '  GET_FILEEX_INFO_LEVELS fInfoLevelId,
    '  LPVOID lpFileInformation
    ');
    Private Declare Auto Function GetFileAttributesEx Lib "kernel32" ( _
        ByVal lpFileName As String, _
        ByVal fInfoLevelId As Integer, _
        ByRef lpFileInformatin As WIN32_FILE_ATTRIBUTE_DATA _
    ) As Boolean

    'HANDLE FindFirstFile(
    '  LPCTSTR lpFileName,
    '  LPWIN32_FIND_DATA lpFindFileData
    ');
    Protected Declare Auto Function FindFirstFile Lib "kernel32" ( _
        ByVal lpFileName As String, _
        ByRef lpFineFileData As WIN32_FIND_DATA _
    ) As IntPtr

    'BOOL FindNextFile(
    '  HANDLE hFindFile,
    '  LPWIN32_FIND_DATA lpFindFileData
    ');
    Protected Declare Auto Function FindNextFile Lib "kernel32" ( _
        ByVal hFindFile As IntPtr, _
        ByRef lpFineFileData As WIN32_FIND_DATA _
    ) As Boolean

    'BOOL FindClose(
    '  HANDLE hFindFile
    ');
    Protected Declare Function FindClose Lib "kernel32" ( _
        ByVal hFindFile As IntPtr _
    ) As Boolean

    'BOOL SetFileTime(
    '  HANDLE hFile,
    '  const FILETIME* lpCreationTime,
    '  const FILETIME* lpLastAccessTime,
    '  const FILETIME* lpLastWriteTime
    ');

    'HANDLE CreateFile(
    '  LPCTSTR lpFileName,
    '  DWORD dwDesiredAccess,
    '  DWORD dwShareMode,
    '  LPSECURITY_ATTRIBUTES lpSecurityAttributes,
    '  DWORD dwCreationDisposition,
    '  DWORD dwFlagsAndAttributes,
    '  HANDLE hTemplateFile
    ');

    'BOOL CloseHandle(
    '  HANDLE hObject
    ');

    'FILE_FLAG_BACKUP_SEMANTICS
    'OPEN_EXISTING
    'FILE_SHARE_READ
    'FILE_SHARE_WRITE
    'GENERIC_READ
    'GENERIC_WRITE
#End Region

    Protected Friend StartingValue As String
    Protected Friend fd As WIN32_FIND_DATA
    Protected LastError As Integer = 0

    Public Sub New()

    End Sub

    Public Sub New(ByVal Start As String)
        Dim hFind As IntPtr
        Dim SearchPath As String
        StartingValue = Start.TrimEnd("\"c)

        ' FindFirstFile has a problem with the Root Directory, so we
        ' use the GetFileAttributesEx API and copy the entries
        If StartingValue.Length = 2 AndAlso StartingValue.EndsWith(":") Then
            Dim fad As New WIN32_FILE_ATTRIBUTE_DATA

            If GetFileAttributesEx(StartingValue & "\", 0, fad) = False Then
                LastError = Marshal.GetLastWin32Error()
                fd = Nothing
                Exit Sub
            End If

            ' populate the structure "by hand"
            fd = New WIN32_FIND_DATA
            fd.cFileName = "\"
            fd.cAlternateFileName = "\"
            fd.dwFileAttributes = fad.dwFileAttributes
            fd.ftCreationTime = fad.ftCreationTime
            fd.ftLastAccessTime = fad.ftLastAccessTime
            fd.ftLastWriteTime = fad.ftLastWriteTime
            fd.nFileSizeHigh = fad.nFileSizeHigh
            fd.nFileSizeLow = fad.nFileSizeLow
            Exit Sub
        End If

        If StartingValue.StartsWith("\\") Then
            SearchPath = "\\?\UNC" & StartingValue.Substring(1)
        Else
            SearchPath = "\\?\" & StartingValue
        End If

        hFind = FindFirstFile(SearchPath, fd)
        If hFind.ToInt32 = INVALID_HANDLE_VALUE Then
            ' strange that this doesn't throw an exception here
            LastError = Marshal.GetLastWin32Error()
            fd = Nothing
        End If
        FindClose(hFind)
    End Sub

    Public ReadOnly Property Attributes() As System.IO.FileAttributes
        Get
            If LastError <> 0 Then
                Return CType(-1, System.IO.FileAttributes)
            Else
                Return CType(fd.dwFileAttributes, System.IO.FileAttributes)
            End If
        End Get
    End Property

    Public ReadOnly Property CreationTime() As DateTime
        Get
            Return Date.FromFileTime((CLng(fd.ftCreationTime.dwHighDateTime) << 32) + fd.ftCreationTime.dwLowDateTime)
        End Get
    End Property

    Public ReadOnly Property CreationTimeUtc() As DateTime
        Get
            Return Date.FromFileTimeUtc((CLng(fd.ftCreationTime.dwHighDateTime) << 32) + fd.ftCreationTime.dwLowDateTime)
        End Get
    End Property

    Public ReadOnly Property Exists() As Boolean
        Get
            Return CBool(LastError = 0)
        End Get
    End Property

    Public ReadOnly Property Extension() As String
        Get
            Return PathEx.GetExtension(FullName)
        End Get
    End Property

    Public ReadOnly Property FullName() As String
        Get
            Return PathEx.GetFullPath(StartingValue)
        End Get
    End Property

    Public ReadOnly Property LastAccessTime() As DateTime
        Get
            Return Date.FromFileTime((CLng(fd.ftLastAccessTime.dwHighDateTime) << 32) + fd.ftLastAccessTime.dwLowDateTime)
        End Get
    End Property

    Public ReadOnly Property LastAccessTimeUtc() As DateTime
        Get
            Return Date.FromFileTimeUtc((CLng(fd.ftLastAccessTime.dwHighDateTime) << 32) + fd.ftLastAccessTime.dwLowDateTime)
        End Get
    End Property

    Public ReadOnly Property LastWriteTime() As DateTime
        Get
            Return Date.FromFileTime((CLng(fd.ftLastWriteTime.dwHighDateTime) << 32) + fd.ftLastWriteTime.dwLowDateTime)
        End Get
    End Property

    Public ReadOnly Property LastWriteTimeUtc() As DateTime
        Get
            Return Date.FromFileTimeUtc((CLng(fd.ftLastWriteTime.dwHighDateTime) << 32) + fd.ftLastWriteTime.dwLowDateTime)
        End Get
    End Property

    Public ReadOnly Property Name() As String
        Get
            Return fd.cFileName
        End Get
    End Property

    ' overridden to provide the file/directory name
    Public Overrides Function ToString() As String
        Return StartingValue
    End Function
End Class

'
' The replacement System.IO.DirectoryInfo class
'
Public Class DirectoryInfoEx
    Inherits FileSystemInfoEx

    Public Sub New()

    End Sub

    Public Sub New(ByVal Start As String)
        MyBase.New(Start)
    End Sub

    Public ReadOnly Property Parent() As DirectoryInfoEx
        Get
            Return New DirectoryInfoEx(PathEx.GetDirectoryName(MyBase.FullName))
        End Get
    End Property

    Public ReadOnly Property Root() As DirectoryInfoEx
        Get
            Return New DirectoryInfoEx(PathEx.GetPathRoot(MyBase.FullName))
        End Get
    End Property

    Public Function GetAccessControl() As DirectorySecurityEx
        Return GetAccessControl(AccessControlSections.All)
    End Function

    Public Function GetAccessControl(ByVal ACSection As AccessControlSections) As DirectorySecurityEx
        Return New DirectorySecurityEx(StartingValue, ACSection)
    End Function

    Public Function GetDirectories() As DirectoryInfoEx()
        Return GetDirectories("", System.IO.SearchOption.TopDirectoryOnly)
    End Function

    Public Function GetDirectories(ByVal Filter As String) As DirectoryInfoEx()
        Return GetDirectories(Filter, System.IO.SearchOption.TopDirectoryOnly)
    End Function

    Public Function GetDirectories(ByVal Filter As String, ByVal Search As System.IO.SearchOption) As DirectoryInfoEx()
        Dim ans(-1) As DirectoryInfoEx
        Dim ldi As New List(Of DirectoryInfoEx)

        SearchForDirectories(StartingValue, Filter.ToUpper, Search, ldi)

        If ldi.Count > 0 Then
            ReDim ans(ldi.Count - 1)
            ldi.CopyTo(ans, 0)
        End If
        Return ans
    End Function

    ' Private routine for doing a recursive search
    Private Sub SearchForDirectories(ByVal DirPath As String, ByVal Filter As String, ByVal Search As System.IO.SearchOption, ByRef ldi As List(Of DirectoryInfoEx))
        Dim fd_temp As New WIN32_FIND_DATA
        Dim hFind As IntPtr
        Dim di As DirectoryInfoEx
        Dim SearchPath As String

        If DirPath.StartsWith("\\") Then
            SearchPath = "\\?\UNC" & DirPath.Substring(1) & "\*"
        Else
            SearchPath = "\\?\" & DirPath & "\*"
        End If

        hFind = FindFirstFile(SearchPath, fd_temp)
        If hFind.ToInt32 <> INVALID_HANDLE_VALUE Then
            Do
                If fd_temp.cFileName = "." Or fd_temp.cFileName = ".." Then
                    Continue Do
                End If

                ' directories only
                If (fd_temp.dwFileAttributes And System.IO.FileAttributes.Directory) <> 0 Then
                    If Filter = "" OrElse fd_temp.cFileName.Equals(Filter, StringComparison.CurrentCultureIgnoreCase) Or fd_temp.cFileName.ToUpper Like Filter Then
                        ' Since we already have the WIN32_FIND_DATA, we pass it on
                        ' rather than building it from scratch
                        di = New DirectoryInfoEx()
                        di.StartingValue = DirPath & "\" & fd_temp.cFileName
                        di.fd = fd_temp
                        ldi.Add(di)
                    End If

                    If Search = System.IO.SearchOption.AllDirectories Then
                        SearchForDirectories(DirPath & "\" & fd_temp.cFileName, Filter, Search, ldi)
                    End If
                End If
            Loop While (FindNextFile(hFind, fd_temp))
            FindClose(hFind)
        End If
    End Sub

    Public Function GetFiles() As FileInfoEx()
        Return GetFiles("", System.IO.SearchOption.TopDirectoryOnly)
    End Function

    Public Function GetFiles(ByVal Filter As String) As FileInfoEx()
        Return GetFiles(Filter, System.IO.SearchOption.TopDirectoryOnly)
    End Function

    Public Function GetFiles(ByVal Filter As String, ByVal Search As System.IO.SearchOption) As FileInfoEx()
        Dim ans(-1) As FileInfoEx
        Dim lfi As New List(Of FileInfoEx)

        SearchForFiles(StartingValue, Filter.ToUpper, Search, lfi)

        If lfi.Count > 0 Then
            ReDim ans(lfi.Count - 1)
            lfi.CopyTo(ans, 0)
        End If
        Return ans
    End Function

    ' A private routine for recursively searching for files
    Private Sub SearchForFiles(ByVal DirPath As String, ByVal Filter As String, ByVal Search As System.IO.SearchOption, ByRef lfi As List(Of FileInfoEx))
        Dim fd_temp As New WIN32_FIND_DATA
        Dim hFind As IntPtr
        Dim fi As FileInfoEx
        Dim SearchPath As String

        If DirPath.StartsWith("\\") Then
            SearchPath = "\\?\UNC" & DirPath.Substring(1) & "\*"
        Else
            SearchPath = "\\?\" & DirPath & "\*"
        End If

        hFind = FindFirstFile(SearchPath, fd_temp)
        If hFind.ToInt32 <> INVALID_HANDLE_VALUE Then
            Do
                If fd_temp.cFileName = "." Or fd_temp.cFileName = ".." Then
                    Continue Do
                End If

                If (fd_temp.dwFileAttributes And System.IO.FileAttributes.Directory) = 0 Then
                    If Filter = "" OrElse fd_temp.cFileName.Equals(Filter, StringComparison.CurrentCultureIgnoreCase) Or fd_temp.cFileName.ToUpper Like Filter Then
                        ' Since we already have the WIN32_FIND_DATA, we pass it on
                        fi = New FileInfoEx()
                        fi.StartingValue = DirPath & "\" & fd_temp.cFileName
                        fi.fd = fd_temp
                        lfi.Add(fi)
                    End If
                Else
                    If Search = System.IO.SearchOption.AllDirectories Then
                        SearchForFiles(DirPath & "\" & fd_temp.cFileName, Filter, Search, lfi)
                    End If
                End If
            Loop While (FindNextFile(hFind, fd_temp))
            FindClose(hFind)
        End If
    End Sub

    Public Function GetFileSystemInfos() As FileSystemInfoEx()
        Return GetFileSystemInfos("")
    End Function

    ' Not recursive
    Public Function GetFileSystemInfos(ByVal Filter As String) As FileSystemInfoEx()
        Dim ans(-1) As FileSystemInfoEx
        Dim fsi As New List(Of FileSystemInfoEx)
        Dim fd_temp As New WIN32_FIND_DATA
        Dim hFind As IntPtr
        Dim di As DirectoryInfoEx
        Dim fi As FileInfoEx
        Dim SearchPath As String

        Filter = Filter.ToUpper
        If StartingValue.StartsWith("\\") Then
            SearchPath = "\\?\UNC" & StartingValue.Substring(1) & "\*"
        Else
            SearchPath = "\\?\" & StartingValue & "\*"
        End If

        hFind = FindFirstFile(SearchPath, fd_temp)
        If hFind.ToInt32 <> INVALID_HANDLE_VALUE Then
            Do
                If fd_temp.cFileName = "." Or fd_temp.cFileName = ".." Then
                    Continue Do
                End If

                If Filter = "" OrElse fd_temp.cFileName.Equals(Filter, StringComparison.CurrentCultureIgnoreCase) Or fd_temp.cFileName.ToUpper Like Filter Then
                    ' fill the generic list with either DirectoryInfoEx or FileInfoEx objects
                    If (fd_temp.dwFileAttributes And System.IO.FileAttributes.Directory) <> 0 Then
                        di = New DirectoryInfoEx()
                        di.StartingValue = StartingValue & "\" & fd_temp.cFileName
                        di.fd = fd_temp
                        fsi.Add(di)
                    Else
                        fi = New FileInfoEx()
                        fi.StartingValue = StartingValue & "\" & fd_temp.cFileName
                        fi.fd = fd_temp
                        fsi.Add(fi)
                    End If
                End If
            Loop While (FindNextFile(hFind, fd_temp))
            FindClose(hFind)
        End If

        If fsi.Count > 0 Then
            ReDim ans(fsi.Count - 1)
            fsi.CopyTo(ans, 0)
        End If
        Return ans
    End Function
End Class

'
' The replacement System.IO.FileInfo class
'
Public Class FileInfoEx
    Inherits FileSystemInfoEx

    Public Sub New()

    End Sub

    Public Sub New(ByVal Start As String)
        MyBase.New(Start)
    End Sub

    Public ReadOnly Property Directory() As DirectoryInfoEx
        Get
            Return New DirectoryInfoEx(PathEx.GetDirectoryName(MyBase.FullName))
        End Get
    End Property

    Public ReadOnly Property DirectoryName() As String
        Get
            Return PathEx.GetDirectoryName(MyBase.FullName)
        End Get
    End Property

    Public ReadOnly Property IsReadonly() As Boolean
        Get
            If LastError <> 0 Then
                ' kinda odd that this defaults to True
                Return True
            Else
                Return CBool(MyBase.Attributes And System.IO.FileAttributes.ReadOnly)
            End If

        End Get
    End Property

    Public ReadOnly Property Length() As Long
        Get
            If LastError <> 0 Then
                ' kinda strange where the exception is thrown
                Throw New Win32Exception(LastError)
            End If
            Return (CLng(fd.nFileSizeHigh) << 32) + fd.nFileSizeLow
        End Get
    End Property

    Public Function GetAccessControl() As FileSecurityEx
        Return GetAccessControl(AccessControlSections.All)
    End Function

    Public Function GetAccessControl(ByVal ACSection As AccessControlSections) As FileSecurityEx
        Return New FileSecurityEx(StartingValue, ACSection)
    End Function
End Class

'
' The replacement for the static System.IO.Directory class
'
Public Class DirectoryEx
    Inherits FileSystemInfoEx

    Public Shared Shadows Function Exists(ByVal DirPath As String) As Boolean
        Dim di As New DirectoryInfoEx(DirPath)
        Return di.Exists
    End Function

    Public Shared Shadows Function GetAccessControl(ByVal DirPath As String) As DirectorySecurityEx
        Dim di As New DirectoryInfoEx(DirPath)
        Return di.GetAccessControl(AccessControlSections.All)
    End Function

    Public Shared Shadows Function GetAccessControl(ByVal DirPath As String, ByVal Section As AccessControlSections) As DirectorySecurityEx
        Dim di As New DirectoryInfoEx(DirPath)
        Return di.GetAccessControl(Section)
    End Function

    Public Shared Function GetDirectories(ByVal DirPath As String) As String()
        Return GetDirectories(DirPath, "", System.IO.SearchOption.TopDirectoryOnly)
    End Function

    Public Shared Function GetDirectories(ByVal DirPath As String, ByVal Filter As String) As String()
        Return GetDirectories(DirPath, Filter, System.IO.SearchOption.TopDirectoryOnly)
    End Function

    Public Shared Function GetDirectories(ByVal DirPath As String, ByVal Filter As String, ByVal Search As System.IO.SearchOption) As String()
        Dim ans(-1) As String
        Dim sc As New StringCollection

        SearchForDirectories(DirPath, Filter.ToUpper, Search, sc)

        If sc.Count > 0 Then
            ReDim ans(sc.Count - 1)
            sc.CopyTo(ans, 0)
        End If
        Return ans
    End Function

    ' Private routine for doing a recursive search
    Private Shared Sub SearchForDirectories(ByVal DirPath As String, ByVal Filter As String, ByVal Search As System.IO.SearchOption, ByRef sc As StringCollection)
        Dim fd_temp As New WIN32_FIND_DATA
        Dim hFind As IntPtr
        Dim SearchPath As String

        If DirPath.StartsWith("\\") Then
            SearchPath = "\\?\UNC" & DirPath.Substring(1) & "\*"
        Else
            SearchPath = "\\?\" & DirPath & "\*"
        End If

        hFind = FindFirstFile(SearchPath, fd_temp)
        If hFind.ToInt32 <> INVALID_HANDLE_VALUE Then
            Do
                If fd_temp.cFileName = "." Or fd_temp.cFileName = ".." Then
                    Continue Do
                End If

                ' directories only
                If (fd_temp.dwFileAttributes And System.IO.FileAttributes.Directory) <> 0 Then
                    If Filter = "" OrElse fd_temp.cFileName.Equals(Filter, StringComparison.CurrentCultureIgnoreCase) Or fd_temp.cFileName.ToUpper Like Filter Then
                        sc.Add(DirPath & "\" & fd_temp.cFileName)
                    End If

                    If Search = System.IO.SearchOption.AllDirectories Then
                        SearchForDirectories(DirPath & "\" & fd_temp.cFileName, Filter, Search, sc)
                    End If
                End If
            Loop While (FindNextFile(hFind, fd_temp))
            FindClose(hFind)
        End If
    End Sub

    ' Similar to DirectoryInfo.GetFiles (except this just returns
    ' a list of file names)
    Public Shared Function GetFiles(ByVal DirPath As String) As String()
        Return GetFiles(DirPath, "", System.IO.SearchOption.TopDirectoryOnly)
    End Function

    Public Shared Function GetFiles(ByVal DirPath As String, ByVal Filter As String) As String()
        Return GetFiles(DirPath, Filter, System.IO.SearchOption.TopDirectoryOnly)
    End Function

    Public Shared Function GetFiles(ByVal DirPath As String, ByVal Filter As String, ByVal Search As System.IO.SearchOption) As String()
        Dim ans(-1) As String
        Dim sc As New StringCollection

        SearchForFiles(DirPath, Filter.ToUpper, Search, sc)

        If sc.Count > 0 Then
            ReDim ans(sc.Count - 1)
            sc.CopyTo(ans, 0)
        End If
        Return ans
    End Function

    ' A private routine for recursively searching for files
    Private Shared Sub SearchForFiles(ByVal DirPath As String, ByVal Filter As String, ByVal Search As System.IO.SearchOption, ByRef sc As StringCollection)
        Dim fd_temp As New WIN32_FIND_DATA
        Dim hFind As IntPtr
        Dim SearchPath As String

        If DirPath.StartsWith("\\") Then
            SearchPath = "\\?\UNC" & DirPath.Substring(1) & "\*"
        Else
            SearchPath = "\\?\" & DirPath & "\*"
        End If

        hFind = FindFirstFile(SearchPath, fd_temp)
        If hFind.ToInt32 <> INVALID_HANDLE_VALUE Then
            Do
                If fd_temp.cFileName = "." Or fd_temp.cFileName = ".." Then
                    Continue Do
                End If

                If (fd_temp.dwFileAttributes And System.IO.FileAttributes.Directory) = 0 Then
                    If Filter = "" OrElse fd_temp.cFileName.Equals(Filter, StringComparison.CurrentCultureIgnoreCase) Or fd_temp.cFileName.ToUpper Like Filter Then
                        sc.Add(DirPath & "\" & fd_temp.cFileName)
                    End If
                Else
                    If Search = System.IO.SearchOption.AllDirectories Then
                        SearchForFiles(DirPath & "\" & fd_temp.cFileName, Filter, Search, sc)
                    End If
                End If
            Loop While (FindNextFile(hFind, fd_temp))
            FindClose(hFind)
        End If
    End Sub

    Public Shared Function GetCreationTime(ByVal DirPath As String) As DateTime
        Dim di As New DirectoryInfoEx(DirPath)
        Return di.CreationTime
    End Function

    Public Shared Function GetCreationTimeUtc(ByVal DirPath As String) As DateTime
        Dim di As New DirectoryInfoEx(DirPath)
        Return di.CreationTimeUtc
    End Function

    Public Shared Function GetDirectoryRoot(ByVal DirPath As String) As String
        Dim di As New DirectoryInfoEx(DirPath)
        Return PathEx.GetPathRoot(di.FullName)
    End Function

    Public Shared Function GetFileSystemEntries(ByVal DirPath As String) As String()
        Return GetFileSystemEntries(DirPath, "")
    End Function

    ' Not recursive
    Public Shared Function GetFileSystemEntries(ByVal DirPath As String, ByVal Filter As String) As String()
        Dim ans(-1), SearchPath As String
        Dim sc As New StringCollection
        Dim fd_temp As New WIN32_FIND_DATA
        Dim hFind As IntPtr

        Filter = Filter.ToUpper
        If DirPath.StartsWith("\\") Then
            SearchPath = "\\?\UNC" & DirPath.Substring(1) & "\*"
        Else
            SearchPath = "\\?\" & DirPath & "\*"
        End If

        hFind = FindFirstFile(SearchPath, fd_temp)
        If hFind.ToInt32 <> INVALID_HANDLE_VALUE Then
            Do
                If fd_temp.cFileName = "." Or fd_temp.cFileName = ".." Then
                    Continue Do
                End If

                If Filter = "" OrElse fd_temp.cFileName.Equals(Filter, StringComparison.CurrentCultureIgnoreCase) Or fd_temp.cFileName.ToUpper Like Filter Then
                    sc.Add(fd_temp.cFileName)
                End If
            Loop While (FindNextFile(hFind, fd_temp))
            FindClose(hFind)
        End If

        If sc.Count > 0 Then
            ReDim ans(sc.Count - 1)
            sc.CopyTo(ans, 0)
        End If
        Return ans
    End Function

    Public Shared Function GetLastAccessTime(ByVal DirPath As String) As DateTime
        Dim di As New DirectoryInfoEx(DirPath)
        Return di.LastAccessTime
    End Function

    Public Shared Function GetLastAccessTimeUtc(ByVal DirPath As String) As DateTime
        Dim di As New DirectoryInfoEx(DirPath)
        Return di.LastAccessTimeUtc
    End Function

    Public Shared Function GetLastWriteTime(ByVal DirPath As String) As DateTime
        Dim di As New DirectoryInfoEx(DirPath)
        Return di.LastWriteTime
    End Function

    Public Shared Function GetLastWriteTimeUtc(ByVal DirPath As String) As DateTime
        Dim di As New DirectoryInfoEx(DirPath)
        Return di.LastWriteTimeUtc
    End Function

    Public Shared Function GetParent(ByVal DirPath As String) As DirectoryInfoEx
        Dim di As New DirectoryInfoEx(DirPath)
        Return di.Parent
    End Function
End Class

'
' The replacement for the static System.IO.File class
'
Public Class FileEx
    Inherits FileSystemInfoEx

    Public Shared Shadows Function Exists(ByVal FilePath As String) As Boolean
        Dim fi As New FileInfoEx(FilePath)
        Return fi.Exists
    End Function

    Public Shared Function GetAttributes(ByVal FilePath As String) As System.IO.FileAttributes
        Dim fi As New FileInfoEx(FilePath)
        Return fi.Attributes
    End Function

    Public Shared Shadows Function GetAccessControl(ByVal FilePath As String) As FileSecurityEx
        Dim fi As New FileInfoEx(FilePath)
        Return fi.GetAccessControl(AccessControlSections.All)
    End Function

    Public Shared Shadows Function GetAccessControl(ByVal FilePath As String, ByVal Section As AccessControlSections) As FileSecurityEx
        Dim fi As New FileInfoEx(FilePath)
        Return fi.GetAccessControl(Section)
    End Function

    Public Shared Function GetCreationTime(ByVal FilePath As String) As DateTime
        Dim fi As New FileInfoEx(FilePath)
        Return fi.CreationTime
    End Function

    Public Shared Function GetCreationTimeUtc(ByVal FilePath As String) As DateTime
        Dim fi As New FileInfoEx(FilePath)
        Return fi.CreationTimeUtc
    End Function

    Public Shared Function GetLastAccessTime(ByVal FilePath As String) As DateTime
        Dim fi As New FileInfoEx(FilePath)
        Return fi.LastAccessTime
    End Function

    Public Shared Function GetLastAccessTimeUtc(ByVal FilePath As String) As DateTime
        Dim fi As New FileInfoEx(FilePath)
        Return fi.LastAccessTimeUtc
    End Function

    Public Shared Function GetLastWriteTime(ByVal FilePath As String) As DateTime
        Dim fi As New FileInfoEx(FilePath)
        Return fi.LastWriteTime
    End Function

    Public Shared Function GetLastWriteTimeUtc(ByVal FilePath As String) As DateTime
        Dim fi As New FileInfoEx(FilePath)
        Return fi.LastWriteTimeUtc
    End Function
End Class

