' I specifically did not import System.IO and System.Security to avoid any
' potential name collisions
Imports System.Collections.ObjectModel
Imports System.Runtime.InteropServices
'
' The replacement System.IO.FileSystemSecurity class that can handle files/paths
' that are greater then 260 characters in length.  This is the base
' class for FileSecurity and DirectorySecurity
'
Public MustInherit Class FileSystemSecurityEx

#Region "API Region"

    'typedef enum _SE_OBJECT_TYPE {
    '  SE_UNKNOWN_OBJECT_TYPE = 0,
    '  SE_FILE_OBJECT,
    '  SE_SERVICE,
    '  SE_PRINTER,
    '  SE_REGISTRY_KEY,
    '  SE_LMSHARE,
    '  SE_KERNEL_OBJECT,
    '  SE_WINDOW_OBJECT,
    '  SE_DS_OBJECT,
    '  SE_DS_OBJECT_ALL,
    '  SE_PROVIDER_DEFINED_OBJECT,
    '  SE_WMIGUID_OBJECT,
    '  SE_REGISTRY_WOW64_32
    '} SE_OBJECT_TYPE;
    Private Enum SE_OBJECT_TYPE As Integer
        SE_UNKNOWN_OBJECT_TYPE = 0
        SE_FILE_OBJECT
        SE_SERVICE
        SE_PRINTER
        SE_REGISTRY_KEY
        SE_LMSHARE
        SE_KERNEL_OBJECT
        SE_WINDOW_OBJECT
        SE_DS_OBJECT
        SE_DS_OBJECT_ALL
        SE_PROVIDER_DEFINED_OBJECT
        SE_WMIGUID_OBJECT
        SE_REGISTRY_WOW64_32
    End Enum

    'typedef struct _ACE_HEADER {
    '  BYTE AceType;
    '  BYTE AceFlags;
    '  WORD AceSize;
    '} ACE_HEADER, *PACE_HEADER;
    <StructLayout(LayoutKind.Sequential)> _
    Structure ACE_HEADER
        Dim AceType As Byte
        Dim AceFlags As Byte
        Dim AceSize As Int16
    End Structure

    'typedef struct _ACCESS_ALLOWED_ACE {
    '  ACE_HEADER Header;
    '  ACCESS_MASK Mask;
    '  DWORD SidStart;
    '} ACCESS_ALLOWED_ACE;
    <StructLayout(LayoutKind.Sequential)> _
    Structure ACCESS_ACE
        Dim Header As ACE_HEADER
        Dim Mask As Integer
        Dim SidStart As Integer
    End Structure

    Private Enum SECURITY_INFORMATION As Integer
        OWNER_SECURITY_INFORMATION = 1
        GROUP_SECURITY_INFORMATION = 2
        DACL_SECURITY_INFORMATION = 4
        SACL_SECURITY_INFORMATION = 8
        PROTECTED_SACL_SECURITY_INFORMATION = 16
        PROTECTED_DACL_SECURITY_INFORMATION = 32
        UNPROTECTED_SACL_SECURITY_INFORMATION = 64
        UNPROTECTED_DACL_SECURITY_INFORMATION = 128
    End Enum

    Public Enum ACE_FLAGS As Byte
        OBJECT_INHERIT_ACE = &H1
        CONTAINER_INHERIT_ACE = &H2
        NO_PROPAGATE_INHERIT_ACE = &H4
        INHERIT_ONLY_ACE = &H8
        INHERITED_ACE = &H10
        VALID_INHERIT_FLAGS = &H1F
        SUCCESSFUL_ACCESS_ACE_FLAG = &H40
        FAILED_ACCESS_ACE_FLAG = &H80
    End Enum

    'DWORD GetNamedSecurityInfo(
    '  LPTSTR pObjectName,
    '  SE_OBJECT_TYPE ObjectType,
    '  SECURITY_INFORMATION SecurityInfo,
    '  PSID* ppsidOwner,
    '  PSID* ppsidGroup,
    '  PACL* ppDacl,
    '  PACL* ppSacl,
    '  PSECURITY_DESCRIPTOR* ppSecurityDescriptor
    ');
    Private Declare Auto Function GetNamedSecurityInfo Lib "advapi32.dll" ( _
        ByVal pObjectName As String, _
        ByVal ObjectType As SE_OBJECT_TYPE, _
        ByVal SecurityInfo As SECURITY_INFORMATION, _
        ByRef ppsidOwner As IntPtr, _
        ByRef ppsidGroup As IntPtr, _
        ByRef ppDacl As IntPtr, _
        ByRef ppSacl As IntPtr, _
        ByRef ppSecurityDescriptor As IntPtr _
    ) As Integer

    'BOOL GetAce(
    '  PACL pAcl,
    '  DWORD dwAceIndex,
    '  LPVOID* pAce
    ');
    Declare Auto Function GetAce Lib "Advapi32.dll" ( _
        ByVal pAcl As IntPtr, _
        ByVal dwAceIndex As Integer, _
        ByRef pAce As IntPtr _
    ) As Boolean

    'BOOL IsValidAcl(
    '  PACL pAcl
    ');
    Declare Auto Function IsValidAcl Lib "Advapi32.dll" ( _
        ByVal pAcl As IntPtr _
    ) As Boolean

    'HLOCAL WINAPI LocalFree(
    '  __in HLOCAL hMem
    ');
    Public Declare Function LocalFree Lib "kernel32" ( _
        ByVal hMem As IntPtr _
    ) As IntPtr

#End Region

    Protected Path As String
    Protected Section As System.Security.AccessControl.AccessControlSections

    Private Structure RawAccessRule
        Public Account As System.Security.Principal.SecurityIdentifier
        Public inheritFlags As System.Security.AccessControl.InheritanceFlags
        Public propFlags As System.Security.AccessControl.PropagationFlags
        Public ruleType As System.Security.AccessControl.AccessControlType
        Public isInherited As Boolean
        Public Mask As Integer
    End Structure

    Public Sub New()

    End Sub

    Public Sub New(ByVal PathName As String, ByVal ACSections As System.Security.AccessControl.AccessControlSections)
        Path = PathName
        Section = ACSections
    End Sub

    Public ReadOnly Property AccessRightType() As Type
        Get
            Return GetType(System.Security.AccessControl.FileSystemRights)
        End Get
    End Property

    Public ReadOnly Property AccessRuleType() As Type
        Get
            Return GetType(System.Security.AccessControl.FileSystemAccessRule)
        End Get
    End Property

    Public ReadOnly Property AuditRuleType() As Type
        Get
            Return GetType(System.Security.AccessControl.FileSystemAuditRule)
        End Get
    End Property

    ' I can't return a real System.Security.AccessControl.AuthorizationRuleCollection, because its
    ' constructors are hidden.  But that's not all bad, since I don't have to backwards compatible
    ' with v1.x and can therefore use a v2.0-style strongly-typed list.
    Public Function GetAccessRules(ByVal includedExplicit As Boolean, ByVal includeInherited As Boolean, ByVal targetType As Type) As ReadOnlyCollection(Of System.Security.AccessControl.FileSystemAccessRule)
        Dim Account As System.Security.Principal.SecurityIdentifier
        Dim inheritFlags As System.Security.AccessControl.InheritanceFlags
        Dim propFlags As System.Security.AccessControl.PropagationFlags
        Dim ruleType As System.Security.AccessControl.AccessControlType
        Dim rules As New List(Of System.Security.AccessControl.FileSystemAccessRule)
        Dim fs As New System.Security.AccessControl.FileSecurity
        Dim ar As System.Security.AccessControl.FileSystemAccessRule
        Dim DupList As New List(Of RawAccessRule)
        Dim Access_ACE As ACCESS_ACE
        Dim SearchPath As String
        Dim isInherited As Boolean
        Dim ace, dacl, sd, sid As IntPtr
        Dim ret, entry As Integer
        Dim raw As RawAccessRule

        If Path.StartsWith("\\") Then
            SearchPath = "\\?\UNC" & Path.Substring(1)
        Else
            SearchPath = "\\?\" & Path
        End If

        ' Get the DACL from inside the Security Descriptor
        ret = GetNamedSecurityInfo(SearchPath, SE_OBJECT_TYPE.SE_FILE_OBJECT, SECURITY_INFORMATION.DACL_SECURITY_INFORMATION, Nothing, Nothing, dacl, Nothing, sd)
        If ret <> 0 Then
            Throw New System.ComponentModel.Win32Exception(ret)
        End If

        ' a quick sanity check...
        If IsValidAcl(dacl) = False Then
            ' Free the memory that got allocated.
            LocalFree(sd)

            ' ERROR_INVALID_ACL
            Throw New System.ComponentModel.Win32Exception(1336)
        End If

        ' loop thru all of the ACE's in the DACL
        entry = 0
        DupList.Clear()
        Do While GetAce(dacl, entry, ace) = True
            ' start by populating the Access_ACE structure
            Access_ACE = CType(Marshal.PtrToStructure(ace, GetType(ACCESS_ACE)), ACCESS_ACE)

            ' Get the sid from the ACE (via some hokey pointer magic)
            sid = New IntPtr(ace.ToInt32 + 8)
            Account = New System.Security.Principal.SecurityIdentifier(sid)

            ' Type of ACE (AceType and AccessControlType are concidently the same values)
            ruleType = CType(Access_ACE.Header.AceType, System.Security.AccessControl.AccessControlType)

            ' Inheritance flags
            inheritFlags = Security.AccessControl.InheritanceFlags.None
            If CBool(Access_ACE.Header.AceFlags And ACE_FLAGS.CONTAINER_INHERIT_ACE) Then
                inheritFlags = inheritFlags Or Security.AccessControl.InheritanceFlags.ContainerInherit
                isInherited = True
            End If
            If CBool(Access_ACE.Header.AceFlags And ACE_FLAGS.OBJECT_INHERIT_ACE) Then
                inheritFlags = inheritFlags Or Security.AccessControl.InheritanceFlags.ObjectInherit
                isInherited = True
            End If

            ' Progagation flags (scope)
            propFlags = Security.AccessControl.PropagationFlags.None
            If CBool(Access_ACE.Header.AceFlags And ACE_FLAGS.NO_PROPAGATE_INHERIT_ACE) Then
                propFlags = propFlags Or Security.AccessControl.PropagationFlags.NoPropagateInherit
            End If
            If CBool(Access_ACE.Header.AceFlags And ACE_FLAGS.INHERIT_ONLY_ACE) Then
                propFlags = propFlags Or Security.AccessControl.PropagationFlags.InheritOnly
            End If

            ' IsInherited
            isInherited = CBool(Access_ACE.Header.AceFlags And ACE_FLAGS.INHERITED_ACE)

            ' should we include this entry?
            If (includedExplicit And Not isInherited) OrElse (includeInherited And isInherited) Then
                '
                ' There is at least one company that uses duplicate ACEs as part of their
                ' copy protection scheme.  This can lead to thousands of duplicate entries
                ' for a single directory.  So, this horrible hack is a way to detect and
                ' record only the unique entries in the DACL.
                '
                raw.Account = Account
                raw.mask = Access_ACE.Mask
                raw.isInherited = isInherited
                raw.inheritFlags = inheritFlags
                raw.propFlags = propFlags
                raw.ruleType = ruleType

                If Not DupList.Contains(raw) Then
                    ' use the real FileSecurity class to build our access rule and add it to the list
                    ar = CType(fs.AccessRuleFactory(Account, Access_ACE.Mask, isInherited, inheritFlags, propFlags, ruleType), System.Security.AccessControl.FileSystemAccessRule)
                    rules.Add(ar)
                    DupList.Add(raw)
                End If
            End If
            entry = entry + 1
        Loop

        ' convert our list into a read-only collection before we pass it back
        Dim ans As New ReadOnlyCollection(Of System.Security.AccessControl.FileSystemAccessRule)(rules)

        ' Free the memory that got allocated.
        LocalFree(sd)

        Return ans
    End Function

    Public Function GetAuditRules(ByVal includedExplicit As Boolean, ByVal includeInherited As Boolean, ByVal targetType As Type) As ReadOnlyCollection(Of System.Security.AccessControl.FileSystemAuditRule)
        Dim rules As New List(Of System.Security.AccessControl.FileSystemAuditRule)
        ' TODO: write this!
        Dim ans As New ReadOnlyCollection(Of System.Security.AccessControl.FileSystemAuditRule)(rules)
        Return ans
    End Function

    Public Function GetGroup(ByVal targetType As Type) As System.Security.Principal.IdentityReference
        Dim sid As System.Security.Principal.SecurityIdentifier
        Dim SearchPath As String
        Dim GroupSid, sd As IntPtr
        Dim ret As Integer

        If Path.StartsWith("\\") Then
            SearchPath = "\\?\UNC" & Path.Substring(1)
        Else
            SearchPath = "\\?\" & Path
        End If

        ' Get the Group's SID.
        ret = GetNamedSecurityInfo(SearchPath, SE_OBJECT_TYPE.SE_FILE_OBJECT, SECURITY_INFORMATION.GROUP_SECURITY_INFORMATION, Nothing, GroupSid, Nothing, Nothing, sd)
        If ret <> 0 Then
            Throw New System.ComponentModel.Win32Exception(ret)
        End If

        sid = New System.Security.Principal.SecurityIdentifier(GroupSid)
        LocalFree(sd)

        If sid.IsValidTargetType(targetType) Then
            If targetType Is GetType(System.Security.Principal.SecurityIdentifier) Then
                Return sid
            End If
            ' Warning: This may throw a ERROR_NONE_MAPPED exception
            Return sid.Translate(targetType)
        Else
            Throw New ArgumentException("Argument must be of type IdentityReference")
        End If
    End Function

    Public Function GetOwner(ByVal targetType As Type) As System.Security.Principal.IdentityReference
        Dim sid As System.Security.Principal.SecurityIdentifier
        Dim SearchPath As String
        Dim OwnerSid, sd As IntPtr
        Dim ret As Integer

        If Path.StartsWith("\\") Then
            SearchPath = "\\?\UNC" & Path.Substring(1)
        Else
            SearchPath = "\\?\" & Path
        End If

        ' Get the Owner's SID.
        ret = GetNamedSecurityInfo(SearchPath, SE_OBJECT_TYPE.SE_FILE_OBJECT, SECURITY_INFORMATION.OWNER_SECURITY_INFORMATION, OwnerSid, Nothing, Nothing, Nothing, sd)
        If ret <> 0 Then
            Throw New System.ComponentModel.Win32Exception(ret)
        End If

        sid = New System.Security.Principal.SecurityIdentifier(OwnerSid)
        LocalFree(sd)

        If sid.IsValidTargetType(targetType) Then
            If targetType Is GetType(System.Security.Principal.SecurityIdentifier) Then
                Return sid
            End If
            ' Warning: This may throw a ERROR_NONE_MAPPED exception
            Return sid.Translate(targetType)
        Else
            Throw New ArgumentException("Argument must be of type IdentityReference")
        End If
    End Function
End Class

Public Class DirectorySecurityEx
    Inherits FileSystemSecurityEx

    Public Sub New()

    End Sub

    Public Sub New(ByVal PathName As String, ByVal ACSections As System.Security.AccessControl.AccessControlSections)
        MyBase.New(PathName, ACSections)
    End Sub
End Class

Public Class FileSecurityEx
    Inherits FileSystemSecurityEx
    Public Sub New()

    End Sub

    Public Sub New(ByVal PathName As String, ByVal ACSections As System.Security.AccessControl.AccessControlSections)
        MyBase.New(PathName, ACSections)
    End Sub
End Class





