Imports System.Data.SqlClient
Imports System.Data

'
' A simple class for dealing with Microsoft SQL Server databases
'
Class SQL_Database
    Private da As New SqlDataAdapter
    Private con As New SqlConnection

    '
    ' create a new table (if it doesn't already exist)
    '
    Public Sub CreateTable(ByVal table_name As String, ByVal DDL As String)
        If TableExists(table_name) = False Then
            Dim cmd As New SqlCommand

            If con.State <> ConnectionState.Open Then
                con.ConnectionString = My.Settings.ConnectString
                con.Open()
            End If

            ' build the new table 
            cmd.CommandText = "Create Table [" & table_name & "]" & DDL
            cmd.Connection = con
            cmd.ExecuteNonQuery()
        End If
    End Sub

    '
    ' Delete a table if it exists
    '
    Public Sub DeleteTable(ByVal table_name As String)
        If TableExists(table_name) Then
            Dim cmd As New SqlCommand

            If con.State <> ConnectionState.Open Then
                con.ConnectionString = My.Settings.ConnectString
                con.Open()
            End If

            ' delete the table completely
            cmd.CommandText = "Drop Table [" & table_name & "]"
            cmd.Connection = con
            cmd.ExecuteNonQuery()
        End If
    End Sub

    '
    ' Empty a table if it exists
    '
    Public Sub EmptyTable(ByVal table_name As String)
        If TableExists(table_name) Then
            Dim cmd As New SqlCommand
            cmd.CommandTimeout = 120

            If con.State <> ConnectionState.Open Then
                con.ConnectionString = My.Settings.ConnectString
                con.Open()
            End If

            ' remove all rows from the table
            cmd.CommandText = "Delete from [" & table_name & "]"
            cmd.Connection = con
            cmd.ExecuteNonQuery()
        End If
    End Sub

    '
    ' Does the table already exist?
    '
    Public Function TableExists(ByVal table_name As String) As Boolean
        Dim cmd As New SqlCommand

        If con.State <> ConnectionState.Open Then
            con.ConnectionString = My.Settings.ConnectString
            con.Open()
        End If

        'cmd.CommandText = "select Count(*) from sys.objects where name = '" & table_name & "' and type_desc = 'USER_TABLE'"
        cmd.CommandText = "Select Count(*) From INFORMATION_SCHEMA.TABLES Where TABLE_TYPE='BASE TABLE' and TABLE_NAME='" & table_name & "'"
        cmd.Connection = con
        If CInt(cmd.ExecuteScalar) > 0 Then
            Return True
        End If

        Return False
    End Function

    '
    ' Open and return a DataTable
    '
    Function OpenTable(ByVal table_name As String) As DataTable
        Dim cmd As New SqlCommand
        Dim dt As New DataTable
        cmd.CommandTimeout = 120

        If con.State <> ConnectionState.Open Then
            con.ConnectionString = My.Settings.ConnectString
            con.Open()
        End If

        cmd.Connection = con
        cmd.CommandText = "Select * from [" & table_name & "]"
        da.SelectCommand = cmd
        da.Fill(dt)
        ' automatically closes the connection

        dt.TableName = table_name
        dt.ExtendedProperties("CommandText") = cmd.CommandText
        Return dt
    End Function

    '
    ' Update the data in the underlying database
    '
    Sub UpdateTable(ByVal dt As DataTable)
        Dim cb As SqlCommandBuilder

        da.SelectCommand.CommandText = dt.ExtendedProperties("CommandText").ToString
        cb = New SqlCommandBuilder(da)
        cb.QuotePrefix = "["
        cb.QuoteSuffix = "]"
        da.Update(dt)
    End Sub

End Class
